package com.iformall.service;

import com.github.pagehelper.PageInfo;
import com.iformall.common.ResultData;
import com.iformall.domain.po.MallUserInfo;
import com.iformall.domain.vo.MallUserInfoVo;

import java.util.List;

public interface MallUserInfoService {

	/**
     * 根据实体查询分页列表
     *
     * @param record
     * @param pageIndex
     * @param pageSize
     * @return
     */
	PageInfo<MallUserInfo> listAsPage(MallUserInfo record, Integer pageIndex, Integer pageSize);

    /**
     * 根据实体查询列表
     * @param record
     * @return
     */
    int cntUserList(MallUserInfo record);
	
	/**
     * 根据Id获得实体
     *
     * @param id
     * @return
     */
    MallUserInfo getById(Long id);
    
        /**
     * 保存或更新实体
     *
     * @param record
     */
    void saveOrUpdate(MallUserInfo record);

    /**
     * 根据Id删除实体
     *
     * @param id
     */
    void deleteById(Long id);

    /**
     * 查询用户名总数，用于校验用户名是否重复
     * @param username
     */
    long cntByUserName(String username);

    /**
     * 根据用户名查询用户对象
     * @param username
     * @return
     */
    MallUserInfo getByUsername(String username);

    /**
     * 检查用户按钮是否有权限
     * @param userId
     * @param permission
     * @return 是否有权限(true: 有| false: 没有)
     */
    boolean hasButtonPermission(Long userId, String permission);


    boolean cntByUserName(String username, Long id);

    boolean cntByUserPhone(String phone, Long id);

    /**
     * 根据phone获得实体
     *
     * @param phone
     * @return
     */
    MallUserInfo getByPhone(String phone, String tenantId);

    /**
     * 根据phone获得实体列表
     *
     * @param phone
     * @return
     */
    List<MallUserInfoVo> getUserByPhone(String phone);

    /**
     * 根据B端OpenId获得实体
     *
     * @param openId
     * @return
     */
    MallUserInfo getByBOpenId(String openId, String tenantId);

    void updateBOpenId(MallUserInfo user);

    /**
     * 根据web端OpenId获得实体
     *
     * @param webOpenId
     * @return
     */
    List<MallUserInfo> getByWebOpenId(String webOpenId);

    /**
     * 根据phone获得展示列表
     *
     * @param webOpenId
     * @return
     */
    List<MallUserInfoVo> getUsersByWebOpenId(String webOpenId);

    /**
     * 根据条件用户列表
     * @return
     */
    List<MallUserInfo> findList(MallUserInfo record);
}
